@@ -1,4 +1,4 @@
-mmer2 <- function(fixed, random, G=NULL, R=NULL, W=NULL, method="NR", REML=TRUE, MVM=FALSE, iters=20, draw=FALSE, init=NULL, data, family=gaussian, silent=FALSE, constraint=TRUE, sherman=FALSE, EIGEND=FALSE, gss=TRUE, forced=NULL, map=NULL, fdr.level=0.05, manh.col=NULL, min.n=FALSE, gwas.plots=TRUE, n.cores=1, tolpar = 1e-06, tolparinv = 1e-06){
+mmer2 <- function(fixed, random, rcov, G=NULL, W=NULL, method="NR", REML=TRUE, MVM=FALSE, iters=20, draw=FALSE, init=NULL, data, family=gaussian, silent=FALSE, constraint=TRUE, sherman=FALSE, EIGEND=FALSE, gss=TRUE, forced=NULL, map=NULL, fdr.level=0.05, manh.col=NULL, min.n=FALSE, gwas.plots=TRUE, n.cores=1, tolpar = 1e-06, tolparinv = 1e-06){
   if(missing(data)){
     data <- environment(fixed)
     data2 <- environment(random)
@@ -28,7 +28,7 @@ mmer2 <- function(fixed, random, G=NULL, R=NULL, W=NULL, method="NR", REML=TRUE,
     data2[,i] <- x
   }
   ######################
-  
+  #print(str(data2))
   mf <- try(model.frame(fixed, data = data2, na.action = na.pass), silent = TRUE)
   mfna <- try(model.frame(fixed, data = data, na.action = na.pass), silent = TRUE)
   if (class(mf) == "try-error") {
@@ -43,8 +43,10 @@ mmer2 <- function(fixed, random, G=NULL, R=NULL, W=NULL, method="NR", REML=TRUE,
   
   #yvar <- gsub(" ", "", as.character(fixed[2]))
   ### Xb in 'formula'
+  #print(str(mf))
+  #print(fixed)
   X <- model.matrix(fixed, mf)
-
+  
   #xvar <- gsub(" ", "", strsplit(as.character(fixed[3]), split = "[+]")[[1]])
   ### Zu in formula
   
@@ -80,12 +82,12 @@ mmer2 <- function(fixed, random, G=NULL, R=NULL, W=NULL, method="NR", REML=TRUE,
       vara <- zvar.names[i]
       # data.frame(factor(V[,vara],levels=V[,vara],ordered=T))
       zi <- model.matrix(as.formula(paste("~",vara,"-1")),zvar)
-
+      
       ## var-cov matrix
       ww <- which(names(G) %in% vara)
       if(length(ww) > 0){# K was provided
         ## just if there's a K matrix we make sure to be using the real names and no the model.matrix ones
-        colnames(zi) <- levels(V[,vara])
+        colnames(zi) <- levels(as.factor(V[,vara]))
         #########
         uuuz <- levels(as.factor(colnames(zi))) # order of Z
         uuuk <- attr(G[[ww]],"dimnames")[[1]] # order of K
@@ -105,6 +107,46 @@ mmer2 <- function(fixed, random, G=NULL, R=NULL, W=NULL, method="NR", REML=TRUE,
     }
     names(Z) <- zvar.names
     
+    if(missing(rcov)){
+      R <- NULL
+    }else{
+      rvar.names <- gsub(" ", "", strsplit(as.character(rcov[2]), split = "[+]")[[1]])
+      R <- list()
+      for(n in 1:length(rvar.names)){ #Ri
+        req.ris <- strsplit(rvar.names[n],":")[[1]] # required Rij's
+        # 1) ar1, 2) cs, 3) arma
+        typ.r <- gsub("\\(.*","",req.ris) # type of correlation matrix
+        typ.v <- gsub(".*\\(","",req.ris); typ.v <- gsub("\\)","",typ.v) # for which variable
+        Ri <- list() # to store Rij's
+        rit <- vector(mode="character")# to store type of correlation matrices
+        for(o in 1:length(typ.r)){#Rij
+          if(typ.r[o]=="ar1"){
+            nr <- length(table(data[,typ.v[o]]))
+            Ri[[o]] <- AR1.mat(.25,nr)
+            rit[o] <- "AR1"
+          }else if(typ.r[o]=="cs"){
+            nr <- length(table(data[,typ.v[o]]))
+            Ri[[o]] <- CS.mat(.25,nr)
+            rit[o] <- "CS"
+          }else if(typ.r[o]=="arma"){
+            nr <- length(table(data[,typ.v[o]]))
+            Ri[[o]] <- AR1.mat(.25,nr)
+            rit[o] <- "AR1"
+          }else if(typ.r[o]=="id"){
+            nr <- length(table(data[,typ.v[o]]))
+            Ri[[o]] <- diag(nr)
+            rit[o] <- "ID"
+          }
+        }
+        ## once we filled Ri put Ri in R
+        R[[n]] <- Ri
+        R[[n]]$type <- rit
+        #names(R[[n]])[o+1] <- "type"
+      }## Ri
+      
+    } # end for rcov present or not
+    #print(str(ETA))
+    #print(str(X))
     res <- mmer(Y=yvar, X=X, Z=Z, R=R, W=W, method=method, REML=REML, iters=iters, draw=draw, init=init, silent=silent, constraint=constraint, sherman=sherman, EIGEND=EIGEND, gss=gss, forced=forced, map=map, fdr.level=fdr.level, manh.col=manh.col,gwas.plots=gwas.plots,n.cores=n.cores, MVM=MVM,tolpar = tolpar, tolparinv = tolparinv)
     
   }else{###only fixed effects
